<?php

class ControllerExtensionShippingFoxpost extends Controller {

	private $error   = array();
	private $version = "1.0.3";

	// INSTALL

	public function install() {
		if (!$this->user->hasPermission('modify', 'extension/extension/payment')) {
            return;
        }

		$this->load->model('extension/shipping/foxpost');
        $this->model_extension_shipping_foxpost->install();
        
        $this->load->model('setting/event');
        $this->model_setting_event->addEvent('shipping_foxpost', 'catalog/model/checkout/order/addOrderHistory/after', 'extension/shipping/foxpost/addOrder');
    }

	// UNINSTALL

	public function uninstall() {
        if (!$this->user->hasPermission('modify', 'extension/extension/shipping')) {
            return;
        }

        $this->load->model('extension/shipping/foxpost');
        $this->model_extension_shipping_foxpost->uninstall();

        $this->load->model('setting/event');
        $this->model_setting_event->deleteEventByCode('shipping_foxpost');
    }

	// INDEX

	public function index() {
		if (!$this->user->hasPermission('access', 'marketplace/extension')) {
			$this->load->language('error/permission');
			$this->document->setTitle($this->language->get('heading_title'));

			$data['header'] = $this->load->controller('common/header');
			$data['column_left'] = $this->load->controller('common/column_left');
			$data['footer'] = $this->load->controller('common/footer');
	
			$this->response->setOutput($this->load->view('error/permission', $data));

			return;
		}

		$this->load->language('extension/shipping/foxpost');
		$this->document->setTitle($this->language->get('heading_title'));

		$this->load->model('setting/setting');

		if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->validate()) {

			$this->model_setting_setting->editSetting('shipping_foxpost', $this->request->post);

			$this->model_setting_setting->editSetting('payment_foxpost', array('payment_foxpost_status' => $this->request->post['shipping_foxpost_status']));

			$this->session->data['success'] = $this->language->get('text_success');

			$this->response->redirect($this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=shipping', true));
		}

		if ($this->config->get('shipping_foxpost_status') && !$this->config->get('payment_foxpost_status')) {
			$this->model_setting_setting->editSetting('payment_foxpost', array('payment_foxpost_status' => $this->config->get('shipping_foxpost_status')));
		}

		$this->load->model('setting/extension');
    	$installed_modules = $this->model_setting_extension->getInstalled('payment');
    	if (!in_array('foxpost', $installed_modules)) {
			$data['error_payment']   = $this->language->get('error_payment_extension');
			$data['extensions_link'] = $this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=payment', true);
    	}

		if (isset($this->error['warning'])) {
			$data['error_warning'] = $this->error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset($this->error['apm_cost'])) {
			$data['error_apm_cost'] = $this->error['apm_cost'];
		} else {
			$data['error_apm_cost'] = '';
		}

		if (isset($this->error['apmcod_cost'])) {
			$data['error_apmcod_cost'] = $this->error['apmcod_cost'];
		} else {
			$data['error_apmcod_cost'] = '';
		}

		if (isset($this->error['transfer_cost'])) {
			$data['error_transfer_cost'] = $this->error['transfer_cost'];
		} else {
			$data['error_transfer_cost'] = '';
		}

		if (isset($this->error['transfercod_cost'])) {
			$data['error_transfercod_cost'] = $this->error['transfercod_cost'];
		} else {
			$data['error_transfercod_cost'] = '';
		}

		if (isset($this->error['service'])) {
			$data['error_service'] = $this->error['service'];
		} else {
			$data['error_service'] = '';
		}

		if (isset($this->error['username'])) {
			$data['error_username'] = $this->error['username'];
		} else {
			$data['error_username'] = '';
		}

		if (isset($this->error['password'])) {
			$data['error_password'] = $this->error['password'];
		} else {
			$data['error_password'] = '';
		}

		if (isset($this->error['key'])) {
			$data['error_key'] = $this->error['key'];
		} else {
			$data['error_key'] = '';
		}

		// data

		if (isset($this->request->post['shipping_foxpost_freelimit'])) {
			$data['shipping_foxpost_freelimit'] = $this->request->post['shipping_foxpost_freelimit'];
		} else {
			$data['shipping_foxpost_freelimit'] = $this->config->get('shipping_foxpost_freelimit');
		}

		if (isset($this->request->post['shipping_foxpost_apm_cost'])) {
			$data['shipping_foxpost_apm_cost'] = $this->request->post['shipping_foxpost_apm_cost'];
		} else {
			$data['shipping_foxpost_apm_cost'] = $this->config->get('shipping_foxpost_apm_cost');
		}

		if (isset($this->request->post['shipping_foxpost_apmcod_cost'])) {
			$data['shipping_foxpost_apmcod_cost'] = $this->request->post['shipping_foxpost_apmcod_cost'];
		} else {
			$data['shipping_foxpost_apmcod_cost'] = $this->config->get('shipping_foxpost_apmcod_cost');
		}

		if (isset($this->request->post['shipping_foxpost_transfer_cost'])) {
			$data['shipping_foxpost_transfer_cost'] = $this->request->post['shipping_foxpost_transfer_cost'];
		} else {
			$data['shipping_foxpost_transfer_cost'] = $this->config->get('shipping_foxpost_transfer_cost');
		}

		if (isset($this->request->post['shipping_foxpost_transfercod_cost'])) {
			$data['shipping_foxpost_transfercod_cost'] = $this->request->post['shipping_foxpost_transfercod_cost'];
		} else {
			$data['shipping_foxpost_transfercod_cost'] = $this->config->get('shipping_foxpost_transfercod_cost');
		}

		if (isset($this->request->post['shipping_foxpost_service'])) {
			$data['shipping_foxpost_service'] = $this->request->post['shipping_foxpost_service'];
		} else {
			$data['shipping_foxpost_service'] = $this->config->get('shipping_foxpost_service');
		}

		if (isset($this->request->post['shipping_foxpost_sort_order'])) {
			$data['shipping_foxpost_sort_order'] = $this->request->post['shipping_foxpost_sort_order'];
		} else {
			$data['shipping_foxpost_sort_order'] = $this->config->get('shipping_foxpost_sort_order');
		}

		if (isset($this->request->post['shipping_foxpost_status'])) {
			$data['shipping_foxpost_status'] = $this->request->post['shipping_foxpost_status'];
		} else {
			$data['shipping_foxpost_status'] = $this->config->get('shipping_foxpost_status');
		}

		if (isset($this->request->post['shipping_foxpost_username'])) {
			$data['shipping_foxpost_username'] = $this->request->post['shipping_foxpost_username'];
		} else {
			$data['shipping_foxpost_username'] = $this->config->get('shipping_foxpost_username');
		}

		if (isset($this->request->post['shipping_foxpost_password'])) {
			$data['shipping_foxpost_password'] = $this->request->post['shipping_foxpost_password'];
		} else {
			$data['shipping_foxpost_password'] = $this->config->get('shipping_foxpost_password');
		}

		if (isset($this->request->post['shipping_foxpost_key'])) {
			$data['shipping_foxpost_key'] = $this->request->post['shipping_foxpost_key'];
		} else {
			$data['shipping_foxpost_key'] = $this->config->get('shipping_foxpost_key');
		}

		$this->load->model('localisation/order_status');

		$data['order_statuses'] = $this->model_localisation_order_status->getOrderStatuses();
	
		if (isset($this->request->post['shipping_foxpost_order_status_id'])) {
			$data['shipping_foxpost_order_status_id'] = $this->request->post['shipping_foxpost_order_status_id'];
		} else {
			$data['shipping_foxpost_order_status_id'] = $this->config->get('shipping_foxpost_order_status_id');
		}

		foreach (['CREATE', 'OPEROUT', 'OPERIN', 'C2CIN', 'C2OUT', 'SORTIN'] as $item) {
			if (isset($this->request->post['shipping_foxpost_order_foxpost_'.$item.'_status_id'])) {
				$data['shipping_foxpost_order_foxpost_'.$item.'_status_id'] = $this->request->post['shipping_foxpost_order_foxpost_'.$item.'_status_id'];
			} else {
				$data['shipping_foxpost_order_foxpost_'.$item.'_status_id'] = $this->config->get('shipping_foxpost_order_foxpost_'.$item.'_status_id');
			}
		}

		$data['apms'] = json_decode(file_get_contents("https://cdn.foxpost.hu/apms.json"), true);

		usort($data['apms'], function($a, $b) {
			return $a['group'] <=> $b['group'];
		});

		$ret = $this->apicall("/api/address");

		$data['api_status'] = false;
		if ($ret['code'] == 200) {
			$data['api_status'] = true;
		}
		else {
			if (isset($ret['data']['error'])) {
				$data['api_error'] = $ret['data']['error'];
			}
			else {
				$data['api_error'] = "CONNECTION ERROR";
			}
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_home'),
			'href' => $this->url->link('common/dashboard', 'user_token=' . $this->session->data['user_token'], true)
		);

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_settings'),
			'href' => $this->url->link('extension/shipping/foxpost', 'user_token=' . $this->session->data['user_token'], true)
		);

		$data['action'] = $this->url->link('extension/shipping/foxpost', 'user_token=' . $this->session->data['user_token'], true);

		$data['cancel'] = $this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=shipping', true);

		$data['header'] = $this->load->controller('common/header');
		$data['column_left'] = $this->load->controller('common/column_left');
		$data['footer'] = $this->load->controller('common/footer');

		$this->response->setOutput($this->load->view('extension/shipping/foxpost_settings', $data));
	}
	
	// VALIDATE SETTINGS

	protected function validate() {
		if (!$this->user->hasPermission('modify', 'extension/shipping/foxpost')) {
			$this->error['warning'] = $this->language->get('error_permission');
		}

		if (!isset($this->request->post['shipping_foxpost_service'])) {
			$this->error['service'] = $this->language->get('error_service');
			
			return !$this->error;
		}

		if (!$this->request->post['shipping_foxpost_username']) {
			$this->error['username'] = $this->language->get('error_username');
		}

		if (!$this->request->post['shipping_foxpost_password']) {
			$this->error['password'] = $this->language->get('error_password');
		}

		if (!$this->request->post['shipping_foxpost_key']) {
			$this->error['key'] = $this->language->get('error_key');
		}

		if (!$this->request->post['shipping_foxpost_apm_cost'] && in_array('apm', @$this->request->post['shipping_foxpost_service'])) {
			$this->error['apm_cost'] = $this->language->get('error_apm_cost');
		}

		if (!$this->request->post['shipping_foxpost_apmcod_cost'] && in_array('apmcod', @$this->request->post['shipping_foxpost_service'])) {
			$this->error['apmcod_cost'] = $this->language->get('error_apmcod_cost');
		}

		if (!$this->request->post['shipping_foxpost_transfer_cost'] && in_array('transfer', @$this->request->post['shipping_foxpost_service'])) {
			$this->error['transfer_cost'] = $this->language->get('error_transfer_cost');
		}

		if (!$this->request->post['shipping_foxpost_transfercod_cost'] && in_array('transfercod', @$this->request->post['shipping_foxpost_service'])) {
			$this->error['transfercod_cost'] = $this->language->get('error_transfercod_cost');
		}

		return !$this->error;
	}

	// ORDERS

	public function orders() {
		if (!$this->user->hasPermission('access', 'sale/order')) {
			$this->load->language('error/permission');
			$this->document->setTitle($this->language->get('heading_title'));

			$data['header'] = $this->load->controller('common/header');
			$data['column_left'] = $this->load->controller('common/column_left');
			$data['footer'] = $this->load->controller('common/footer');
	
			$this->response->setOutput($this->load->view('error/permission', $data));

			return;
		}

		$this->load->language('extension/shipping/foxpost');
		$this->document->setTitle($this->language->get('heading_title'));

		$get = $this->request->get;

		// TEST

		if (isset($get['test'])) { 

			exit;
		}

		// MIGRATE

		if (isset($get['migrate'])) { 
			$query         = $this->db->query("INSERT INTO " . DB_PREFIX . "foxpost_order (order_id, foxpost_type, foxpost_barcode, date_added) SELECT order_id, shipping_code, tracking, date_added FROM " . DB_PREFIX . "order WHERE order_id NOT IN ( SELECT order_id FROM " . DB_PREFIX . "foxpost_order )" );
			$foxpost_datas = $query->rows;

			print "<pre>";
			print_r($foxpost_datas);
			exit;
		}

		// EXPORT

		elseif (isset($get['export'])) {
			$query = $this->db->query("SELECT * FROM " . DB_PREFIX . "foxpost_order WHERE foxpost_order_id IN(" . $get['export'] . ')' );
			$temp = $query->rows;

			$ids = [];
			foreach ($temp as $item) {
				$ids[] = $item['order_id'];
				$foxpost_data[$item['order_id']] = $item;
			}

			if ($ids) {
				$query = $this->db->query("SELECT * FROM " . DB_PREFIX . "order WHERE order_id IN(" . implode(',', $ids) . ')' );
				$temp = $query->rows;

				foreach ($temp as $item) {
					$order_data[$item['order_id']] = $item;
				}
			}

			$headers = array(
				"Címzett neve",
				"Címzett telefonszáma",
				"Címzett email címe",
				"Átvételi automata",
				"Település",
				"Irányítószám",
				"Utca, házszám",
				"Utánvételi összeg",
				"Csomag méret",
				"Futár információk",
				"Saját adatok",
				"Címkenyomtatás",
				"Törékeny",
				"Egyedi vonalkód",
				"Referencia kód"
			);

			$out = '"'.implode('","', $headers).'"'."\n";

			foreach ($foxpost_data as $foxpost_row) {
				$row_data   = array();
				$order_row = $order_data[$foxpost_row['order_id']];

				$delivery = explode(".", $foxpost_row['foxpost_type']);

				$total = round($order_row['total']);

				$row_data[] = $order_row['lastname']." ".$order_row['firstname'];
				$row_data[] = $order_row['telephone'];
				$row_data[] = $order_row['email'];

				if (isset($delivery[1]) && $delivery[1] != 'transfer' && $delivery[1] != 'transfercod') {
					$apm = explode("_", $delivery[1]);

					$row_data[] = $apm[0]; // Átvételi automata
					$row_data[] = ''; // Település
					$row_data[] = ''; // Irányítószám
					$row_data[] = ''; // Utca, házszám

					if (isset($apm[1]) && $apm[1] == "cod") {
						$row_data[] = $total; // Utánvételi összeg
					}
					else {
						$row_data[] = ''; // Utánvételi összeg
					}
				}
				else {
					$row_data[] = ''; // Átvételi automata
					$row_data[] = $order_row['shipping_city']; // Település
					$row_data[] = $order_row['shipping_postcode']; // Irányítószám
					$row_data[] = $order_row['shipping_address_1'].' '.$order_row['shipping_address_2']; // Utca, házszám

					if ($delivery[1] == "transfercod") {
						$row_data[] = $total; // Utánvételi összeg
					}
					else {
						$row_data[] = ''; // Utánvételi összeg
					}
				}

				$row_data[] = $foxpost_row['foxpost_size']?$foxpost_row['foxpost_size']:strtoupper($get['size']); // Csomag méret
				$row_data[] = str_replace("\n", " ", $order_row['comment']); // Futár információk
				$row_data[] = ''; // Saját adatok
				$row_data[] = ''; // Címkenyomtatás
				$row_data[] = ''; // Törékeny
				$row_data[] = $foxpost_row['foxpost_barcode']; // Egyedi vonalkód
				$row_data[] = ''; // Referencia kód

				$out .= '"'.implode('","', $row_data).'"'."\n";
			}

			$this->response->addHeader('Content-Type: text/csv');
			$this->response->addHeader('Content-Disposition: attachment; filename=foxpost_export.csv');
			$this->response->setOutput($out);
		}

		// BARCODE

		elseif (isset($get['barcode']) && isset($get['pagesize'])) { 
			$query         = $this->db->query("SELECT * FROM " . DB_PREFIX . "foxpost_order WHERE foxpost_barcode != '' AND foxpost_order_id IN(" . $get['barcode'] . ')' );

			$foxpost_datas = $query->rows;

			$data  = [];
			$error = '';

			foreach ($foxpost_datas as $foxpost_data) {
				$data[] = $foxpost_data['foxpost_barcode'];
			}

			$resp = $this->apicall('/api/label/'.$get['pagesize'], 'POST', $data);

			$this->response->addHeader('Content-Type: application/pdf');
			$this->response->setOutput($resp['data']);
		}

		// SYNC

		elseif (isset($get['sync'])) { 

			$query         = $this->db->query("SELECT * FROM " . DB_PREFIX . "foxpost_order WHERE foxpost_barcode != '' AND foxpost_order_id IN(" . $get['sync'] . ')' );
			$foxpost_datas = $query->rows;

			$data  = [];
			$error = '';

			foreach ($foxpost_datas as $foxpost_data) {
				$resp = $this->apicall('/api/tracking/'.$foxpost_data['foxpost_barcode']);

				if (isset($resp['code']) && $resp['code'] == 200) {
					// OK
				}
				elseif (isset($resp['code']) && $resp['code'] == 401) {
					$error = "FOXPOST API: " . $this->language->get('error_api_401') . " (401)";
				}
				elseif (isset($resp['code']) && $resp['code'] == 400) {
					$error = "FOXPOST API: " . (isset($resp['data']['error'])?$resp['data']['error']:$this->language->get('error_api_unknow') . " (400)");
				}
				else {
					$error = $this->language->get('error_api_unknow');
				}

				if ($error) {
					break;
				}
				else {
					if ($sid = $this->config->get('shipping_foxpost_order_foxpost_'.$resp['data']['traces'][0]['status'].'_status_id')) {
						$query = $this->db->query("
							UPDATE " . DB_PREFIX . "order
							SET
								order_status_id = '" . $sid . "'	
							WHERE
								order_id = " . round($foxpost_data['order_id']) . " AND
								order_status_id != " . $sid
						);

						if ($this->db->countAffected()) {
							$this->load->model('extension/shipping/foxpost');
							$this->model_extension_shipping_foxpost->addOrderHistory(
								round($foxpost_data['order_id']),
								$sid,
								'Foxpost'
							);
						}
					}
					
					$query = $this->db->query("
						UPDATE " . DB_PREFIX . "foxpost_order
						SET
							foxpost_shortname = '" . $resp['data']['traces'][0]['shortName'] . "',
							foxpost_status = '" . $resp['data']['traces'][0]['status'] . "',
							date_modified = NOW()
						WHERE
							foxpost_order_id = " . $foxpost_data['foxpost_order_id']
						);
				}
			}
			exit;

			if ($error) {
				$data = array(
					'message' => $error
				);
				$this->response->addHeader('HTTP/1.1 400 Bad Request');
			}

			$this->response->addHeader('Content-Type: application/json');
			$this->response->setOutput(json_encode($data));
		}

		// CREATE

		elseif (isset($get['create'])) {

			$query = $this->db->query("SELECT * FROM " . DB_PREFIX . "foxpost_order WHERE foxpost_order_id = " . round($get['create']));
			$foxpost_data = $query->row;

			$this->load->model('sale/order');
			$order_data = $this->model_sale_order->getOrder($foxpost_data['order_id']);

			list($temp, $foxpost_type) = explode(".", $foxpost_data['foxpost_type']);

			if ($foxpost_type == "transfer") {
				$data['recipientCity']    = $order_data['shipping_city'];
				$data['recipientZip']     = $order_data['shipping_postcode'];
				$data['recipientAddress'] = $order_data['shipping_address_1']." ".$order_data['shipping_address_2'];
			}
			elseif ($foxpost_type == "transfercod") {
				$data['cod']              = round($order_data['total']);

				$data['recipientCity']    = $order_data['shipping_city'];
				$data['recipientZip']     = $order_data['shipping_postcode'];
				$data['recipientAddress'] = $order_data['shipping_address_1']." ".$order_data['shipping_address_2'];
			}
			else {
				$foxpost_type  = explode("_", $foxpost_type);
				$foxpost_iscod = isset($foxpost_type[1])?$foxpost_type[1]:'';
				$foxpost_type  = $foxpost_type[0];

				if ($foxpost_iscod == "cod") {
					$data['cod'] = round($order_data['total']);
				}

				$data['destination'] = $foxpost_type;
			}

			$data['recipientEmail'] = $order_data['email'];
			$data['recipientPhone'] = $order_data['telephone'];
			$data['recipientName']  = $order_data['shipping_lastname']." ".$order_data['shipping_firstname'];

			$data['size']           = $get['size'];
			$data['source']         = "opencart3_".$this->version;

			$error = '';

			$resp = $this->apicall('/api/parcel', 'POST', [$data]);

			$data = array();

			if (isset($resp['code']) && $resp['code'] == 201) {
				// OK
			}
			elseif (isset($resp['code']) && $resp['code'] == 401) {
				$error = "FOXPOST API: " . $this->language->get('error_api_401') . " (401)";
			}
			elseif (isset($resp['code']) && $resp['code'] == 400) {
				$error = "FOXPOST API: " . (isset($resp['data']['error'])?$resp['data']['error']:$this->language->get('error_api_unknow') . " (400)");
			}
			else {
				$error = $this->language->get('error_api_unknow');
			}

			if (!(isset($resp['data']['valid']) && $resp['data']['valid'])) {
				if (isset($resp['data']['parcels'][0]['errors'][0]['message']) && $resp['data']['parcels'][0]['errors'][0]['message']) {
					$error = "FOXPOST API: " . $resp['data']['parcels'][0]['errors'][0]['message'];
				}
				else {
					$error = "FOXPOST API: " . $this->language->get('error_api_validation');
				}
			}

			if ($error) {
				$data = array(
					'message' => $error
				);
				$this->response->addHeader('HTTP/1.1 400 Bad Request');
			}
			else {
				$data_temp = $data;
				$data = [];

				$query = $this->db->query("UPDATE " . DB_PREFIX . "foxpost_order SET foxpost_barcode = '" . $resp['data']['parcels'][0]['clFoxId'] . "', foxpost_size = '" . $resp['data']['parcels'][0]['size'] . "', date_modified = NOW() WHERE foxpost_order_id = " . round($get['create']) );
				$query = $this->db->query("UPDATE " . DB_PREFIX . "order SET tracking = '" . $resp['data']['parcels'][0]['clFoxId'] . "' WHERE order_id = " . round($foxpost_data['order_id']) );

				if ($this->config->get('shipping_foxpost_order_foxpost_status_id')) {
					$comment = $this->language->get('text_foxpost_tracking') . ": https://foxpost.hu/csomagkovetes/?code=" . $resp['data']['parcels'][0]['clFoxId'];

					$this->load->model('extension/shipping/foxpost');
					$this->model_extension_shipping_foxpost->addOrderHistory(
						round($foxpost_data['order_id']),
						$this->config->get('shipping_foxpost_order_foxpost_status_id'),
						$comment
					);
				}

				$order_info = $order_data;
	
				$language = new Language($order_info['language_code']);
				$language->load($order_info['language_code']);
				$language->load('extension/shipping/foxpost');
	
				$data['order_info']  = $order_info;
	
				$order_info['date_added'] = date($language->get('date_format_short'), strtotime($order_info['date_added']));
	
				$data['logo']          = $order_info['store_url'] . 'image/' . $this->config->get('config_logo');
				$data['store_name']    = $order_info['store_name'];
				$data['store_url']     = $order_info['store_url'];
				$data['customer_id' ]  = $order_info['customer_id'];
				$data['link']          = $order_info['store_url'] . 'index.php?route=account/order/info&order_id=' . $order_info['order_id'];
	
				$data['title']                 = sprintf($language->get('mail_foxpost_subject'), $order_info['store_name'], $order_info['order_id']);
				$data['mail_foxpost_greeting'] = sprintf($language->get('mail_foxpost_greeting'), $order_info['store_name']);
	
				$data['foxpost_data'] = $foxpost_data;

				$data['foxpost_data']['foxpost_barcode'] = $resp['data']['parcels'][0]['clFoxId'];
	
				if ($order_info['payment_address_format']) {
					$format = $order_info['payment_address_format'];
				} else {
					$format = '{firstname} {lastname}' . "\n" . '{company}' . "\n" . '{address_1}' . "\n" . '{address_2}' . "\n" . '{city} {postcode}' . "\n" . '{zone}' . "\n" . '{country}';
				}
		
				$find = array(
					'{firstname}',
					'{lastname}',
					'{company}',
					'{address_1}',
					'{address_2}',
					'{city}',
					'{postcode}',
					'{zone}',
					'{zone_code}',
					'{country}'
				);
		
				$replace = array(
					'firstname' => $order_info['payment_firstname'],
					'lastname'  => $order_info['payment_lastname'],
					'company'   => $order_info['payment_company'],
					'address_1' => $order_info['payment_address_1'],
					'address_2' => $order_info['payment_address_2'],
					'city'      => $order_info['payment_city'],
					'postcode'  => $order_info['payment_postcode'],
					'zone'      => $order_info['payment_zone'],
					'zone_code' => $order_info['payment_zone_code'],
					'country'   => $order_info['payment_country']
				);
		
				$data['payment_address'] = str_replace(array("\r\n", "\r", "\n"), '<br />', preg_replace(array("/\s\s+/", "/\r\r+/", "/\n\n+/"), '<br />', trim(str_replace($find, $replace, $format))));
		
				if ($order_info['shipping_address_format']) {
					$format = $order_info['shipping_address_format'];
				} else {
					$format = '{firstname} {lastname}' . "\n" . '{company}' . "\n" . '{address_1}' . "\n" . '{address_2}' . "\n" . '{city} {postcode}' . "\n" . '{zone}' . "\n" . '{country}';
				}
		
				$find = array(
					'{firstname}',
					'{lastname}',
					'{company}',
					'{address_1}',
					'{address_2}',
					'{city}',
					'{postcode}',
					'{zone}',
					'{zone_code}',
					'{country}'
				);
		
				$replace = array(
					'firstname' => $order_info['shipping_firstname'],
					'lastname'  => $order_info['shipping_lastname'],
					'company'   => $order_info['shipping_company'],
					'address_1' => $order_info['shipping_address_1'],
					'address_2' => $order_info['shipping_address_2'],
					'city'      => $order_info['shipping_city'],
					'postcode'  => $order_info['shipping_postcode'],
					'zone'      => $order_info['shipping_zone'],
					'zone_code' => $order_info['shipping_zone_code'],
					'country'   => $order_info['shipping_country']
				);
		
				$data['shipping_address'] = str_replace(array("\r\n", "\r", "\n"), '<br />', preg_replace(array("/\s\s+/", "/\r\r+/", "/\n\n+/"), '<br />', trim(str_replace($find, $replace, $format))));
	
				$mail = new Mail($this->config->get('config_mail_engine'));
				$mail->parameter     = $this->config->get('config_mail_parameter');
				$mail->smtp_hostname = $this->config->get('config_mail_smtp_hostname');
				$mail->smtp_username = $this->config->get('config_mail_smtp_username');
				$mail->smtp_password = html_entity_decode($this->config->get('config_mail_smtp_password'), ENT_QUOTES, 'UTF-8');
				$mail->smtp_port     = $this->config->get('config_mail_smtp_port');
				$mail->smtp_timeout  = $this->config->get('config_mail_smtp_timeout');
	
				$mail->setTo($order_data['email']);
				$mail->setFrom($this->config->get('config_email'));
				$mail->setSender(html_entity_decode($this->config->get('config_name'), ENT_QUOTES, 'UTF-8'));
				$mail->setSubject(sprintf($this->language->get('mail_foxpost_subject'), html_entity_decode($this->config->get('config_name'), ENT_QUOTES, 'UTF-8'), $foxpost_data['order_id']));
				$mail->setHtml($this->load->view('extension/shipping/foxpost_mail', $data));
				$mail->send();

				$data = $data_temp;

				$this->response->redirect($this->url->link('extension/shipping/foxpost/orders', 'user_token=' . $this->session->data['user_token'].'&sync=' . $foxpost_data['foxpost_order_id'], true));
			}

			// $data['data'] = $resp;

			$this->response->addHeader('Content-Type: application/json');
			$this->response->setOutput(json_encode($data));
		}

		// LIST

		else {

			$data['link_base']    = $this->url->link('extension/shipping/foxpost/orders', 'user_token=' . $this->session->data['user_token'], true);
			$data['link_archive'] = $this->url->link('extension/shipping/foxpost/orders', 'user_token=' . $this->session->data['user_token'] . '&filter=archive', true);

			$orderby    = isset($get['orderby'])?$get['orderby']:'date_added';
			$orderbysub = isset($get['orderbysub'])?$get['orderbysub']:'DESC';

			$data['orderby']    = $orderby;
			$data['orderbysub'] = $orderbysub;

			if (isset($get['filter']) && $get['filter'] == 'archive') {
				$query = $this->db->query("SELECT * FROM " . DB_PREFIX . "foxpost_order WHERE foxpost_status IN ('RECIEVE', 'HDRECIEVE', 'HDRETURN', 'INWAREHOUSE', 'COLLECTSENT') ORDER BY " . $orderby . " " . $orderbysub);
				$data['archive']  = true;
			}
			else {
				$query = $this->db->query("SELECT * FROM " . DB_PREFIX . "foxpost_order WHERE foxpost_status NOT IN ('RECIEVE', 'HDRECIEVE', 'HDRETURN', 'INWAREHOUSE', 'COLLECTSENT') ORDER BY " . $orderby . " " . $orderbysub);
			}

			$data['data'] = $query->rows;

			foreach ($data['data'] as &$item) {
				$item['link_sync']    = $this->url->link('extension/shipping/foxpost/orders', 'user_token=' . $this->session->data['user_token'].'&sync=' .     $item['foxpost_order_id'], true);
				$item['link_barcode'] = $this->url->link('extension/shipping/foxpost/orders', 'user_token=' . $this->session->data['user_token'].'&barcode=' .  $item['foxpost_order_id'], true);
				$item['link_create']  = $this->url->link('extension/shipping/foxpost/orders', 'user_token=' . $this->session->data['user_token'].'&create=' .   $item['foxpost_order_id'], true);
				$item['link_order']   = $this->url->link('sale/order/info',                   'user_token=' . $this->session->data['user_token'].'&order_id=' . $item['order_id'], true);

				list($temp, $foxpost_type) = explode(".", $item['foxpost_type']);

				if ($foxpost_type == "transfer") {
					$item['foxpost_type'] = 'transfer';
					$item['foxpost_cod']  = false;
				}
				elseif ($foxpost_type == "transfercod") {
					$item['foxpost_type'] = 'transfer';
					$item['foxpost_cod']  = true;
				}
				else {
					$foxpost_type  = explode("_", $foxpost_type);
					$foxpost_iscod = isset($foxpost_type[1])?$foxpost_type[1]:'';

					$item['foxpost_type'] = $foxpost_type[0];
	
					if ($foxpost_iscod == "cod") {
						$item['foxpost_cod'] = true;
					}
					else {
						$item['foxpost_cod'] = false;
					}
				}
			}
			
			$ret = $this->apicall("/api/address");

			$data['api_status'] = false;
			if ($ret['code'] == 200) {
				$data['api_status'] = true;
			}

			$data['breadcrumbs'] = array();

			$data['breadcrumbs'][] = array(
				'text' => $this->language->get('text_home'),
				'href' => $this->url->link('common/dashboard', 'user_token=' . $this->session->data['user_token'], true)
			);

			$data['breadcrumbs'][] = array(
				'text' => $this->language->get('text_orders'),
				'href' => $this->url->link('extension/shipping/foxpost/orders', 'user_token=' . $this->session->data['user_token'], true)
			);

			$data['header'] = $this->load->controller('common/header');
			$data['column_left'] = $this->load->controller('common/column_left');
			$data['footer'] = $this->load->controller('common/footer');

			$data['apms'] = [];
			$temp = json_decode(file_get_contents("https://cdn.foxpost.hu/apms.json"), true);
			foreach ($temp as $apm) {
				$data['apms'][$apm['operator_id']] = $apm['name'];
			}
			unset($temp);

			$this->response->setOutput($this->load->view('extension/shipping/foxpost_orders', $data));
		}
    }

	// API CALL

	protected function apicall($path, $method = 'GET', $request_data = array(), $query = array()) {
		$url = "https://webapi.foxpost.hu".$path.($query?'?'.http_build_query($query):'');

		$code    = '';
		$headers = [];

		$username = $this->config->get('shipping_foxpost_username');
		$password = $this->config->get('shipping_foxpost_password');
		$key      = $this->config->get('shipping_foxpost_key');

		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);
		curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($request_data));
		curl_setopt($ch, CURLOPT_HTTPHEADER,
			array(
				"Content-Type: application/json",
				'authorization: Basic ' . base64_encode($username.":".$password),
				'api-key: ' . $key
			)
		);
		curl_setopt($ch, CURLOPT_TIMEOUT, 10);
		curl_setopt($ch, CURLOPT_HEADER, true);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

		$response = curl_exec($ch);

		$code   = curl_getinfo($ch, CURLINFO_HTTP_CODE);
		$header = substr($response, 0, curl_getinfo($ch, CURLINFO_HEADER_SIZE));
		$data   = substr($response, curl_getinfo($ch, CURLINFO_HEADER_SIZE));
		$header = explode("\n", $header);

		foreach ($header as &$header_item) {
			$header_item = trim($header_item);
		}

		if (in_array("Content-Type: application/json", $header) || in_array("Content-Type: application/json;charset=ISO-8859-1", $header)) {
			$data = json_decode($data, true);
		}

		curl_close($ch);

		return array(
			'code'    => $code,
			'header'  => $header,
			'data'    => $data,
			// FOR DEBUG
			'request' => array(
				'url'     => $url,
				'method'  => $method,
				'payload' => $request_data
			)
		);
	}

}
